﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2012 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using System.Linq;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Problems.DataAnalysis {
  /// <summary>
  /// 
  /// </summary>
  [StorableClass]
  [Item("NeighbourhoodWeightCalculator", "")]
  public class NeighbourhoodWeightCalculator : DiscriminantClassificationWeightCalculator {

    public NeighbourhoodWeightCalculator()
      : base() {
    }

    [StorableConstructor]
    protected NeighbourhoodWeightCalculator(bool deserializing) : base(deserializing) { }
    protected NeighbourhoodWeightCalculator(NeighbourhoodWeightCalculator original, Cloner cloner)
      : base(original, cloner) {
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new NeighbourhoodWeightCalculator(this, cloner);
    }

    protected override IEnumerable<double> DiscriminantCalculateWeights(ItemCollection<IDiscriminantFunctionClassificationSolution> discriminantSolutions) {
      List<List<double>> estimatedTrainingValEnumerators = new List<List<double>>();
      List<List<double>> estimatedTrainingClassValEnumerators = new List<List<double>>();
      foreach (var solution in discriminantSolutions) {
        estimatedTrainingValEnumerators.Add(solution.EstimatedTrainingValues.ToList());
        estimatedTrainingClassValEnumerators.Add(solution.EstimatedTrainingClassValues.ToList());
      }

      List<double> weights = Enumerable.Repeat<double>(0, discriminantSolutions.Count()).ToList<double>();

      IClassificationProblemData problemData = discriminantSolutions.ElementAt(0).ProblemData;
      List<double> targetValues = problemData.Dataset.GetDoubleValues(problemData.TargetVariable).ToList();
      List<double> trainingVal = GetValues(targetValues, problemData.TrainingIndizes).ToList();

      double pointAvg, help;
      int count;
      for (int point = 0; point < estimatedTrainingClassValEnumerators.First().Count; point++) {
        pointAvg = 0.0;
        count = 0;
        for (int solution = 0; solution < estimatedTrainingClassValEnumerators.Count; solution++) {
          if (estimatedTrainingClassValEnumerators[solution][point].Equals(targetValues[point])) {
            pointAvg += estimatedTrainingValEnumerators[solution][point];
            count++;
          }
        }
        pointAvg /= (double)count;
        for (int solution = 0; solution < estimatedTrainingClassValEnumerators.Count; solution++) {
          if (estimatedTrainingClassValEnumerators[solution][point].Equals(targetValues[point])) {
            weights[solution] += 0.5;
            help = Math.Abs(estimatedTrainingValEnumerators[solution][point] - 0.5);
            weights[solution] += help < 0.5 ? 0.5 - help : 0.0;
          }
        }
      }
      return weights;
    }
  }
}
